import glob
import mistune
import os
from bs4 import BeautifulSoup
from jinja2 import Environment, PackageLoader, select_autoescape
import click
from bottle import Bottle, run


def extract_statuses_from_adr(page_object):
    status_section = page_object.find('h2', text='Status')

    current_node = status_section.nextSibling

    while current_node.name != 'h2':

        current_node = current_node.nextSibling

        if current_node.name == 'p':
            yield current_node.text
        else:
            continue


def parse_adr_to_config(path):
    adr_as_html = mistune.markdown(open(path).read())

    soup = BeautifulSoup(adr_as_html, features='html.parser')

    status = list(extract_statuses_from_adr(soup))

    if any([line.startswith("Amended by") for line in status]):
        status = 'amended'
    elif any([line.startswith("Accepted") for line in status]):
        status = 'accepted'
    elif any([line.startswith("Superceded by") for line in status]):
        status = 'superceded'
    elif any([line.startswith("Pending") for line in status]):
        status = 'pending'
    else:
        status = 'unknown'

    header = soup.find('h1').text

    return {
        'status': status,
        'body': adr_as_html,
        'title': header
    }


def render_html(config):

    env = Environment(
        loader=PackageLoader('adr_viewer', 'templates'),
        autoescape=select_autoescape(['html', 'xml'])
    )

    template = env.get_template('index.html')

    return template.render(config=config)


def get_adr_files(path):
    files = glob.glob(path)
    files.sort()
    return files


def run_server(content):
    print('Starting server at http://localhost:8000/')
    app = Bottle()
    app.route('/', 'GET', lambda: content)
    run(app, host='localhost', port=8000, quiet=True)


def generate_content(path):

    files = get_adr_files("%s/*.md" % path)

    config = {
        'project_title': os.path.basename(os.getcwd()),
        'records': []
    }

    for index, adr_file in enumerate(files):

        adr_attributes = parse_adr_to_config(adr_file)

        adr_attributes['index'] = index

        config['records'].append(adr_attributes)

    return render_html(config)


@click.command()
@click.option('--adr-path', default='doc/adr/',   help='Directory containing ADR files.',         show_default=True)
@click.option('--output',   default='index.html', help='File to write output to.',                show_default=True)
@click.option('--serve',    default=False,        help='Serve content at http://localhost:8000/', is_flag=True)
def main(adr_path, output, serve):
    content = generate_content(adr_path)

    if serve:
        run_server(content)
    else:
        with open(output, 'w') as out:
            out.write(content)



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='adr-viewer',
    url='https://github.com/mrwilson/adr-viewer',
    version='1.2.0',
    description='A visualisation tool for Architecture Decision Records',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Alex Wilson',
    author_email='a.wilson@alumni.warwick.ac.uk',
    license='MIT',
    packages=find_packages(),
    install_requires=(
        'click',
        'mistune',
        'bs4',
        'jinja2',
        'bottle'
    ),
    package_data={
        'adr_viewer': ['templates/index.html'],
    },
    entry_points={
        'console_scripts': ['adr-viewer=adr_viewer:main']
    },
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Topic :: Software Development',
        'Topic :: Software Development :: Documentation',
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
    ]
)



from adr_viewer import parse_adr_to_config, render_html


def test_should_extract_title_from_record():
    config = parse_adr_to_config('doc/adr/0001-record-architecture-decisions.md')

    assert config['title'] == '1. Record architecture decisions'


def test_should_extract_status_from_record():
    config = parse_adr_to_config('doc/adr/0001-record-architecture-decisions.md')

    assert config['status'] == 'accepted'


def test_should_include_adr_as_html():
    config = parse_adr_to_config('doc/adr/0001-record-architecture-decisions.md')

    assert '<h1>1. Record architecture decisions</h1>' in config['body']


def test_should_mark_superceded_records():
    config = parse_adr_to_config('doc/adr/0003-use-same-colour-for-all-headers.md')

    assert config['status'] == 'superceded'


def test_should_mark_amended_records():
    config = parse_adr_to_config('doc/adr/0004-distinguish-superceded-records-with-colour.md')

    assert config['status'] == 'amended'


def test_should_mark_unknown_records():
    config = parse_adr_to_config('test/adr/0001-unknown-status.md')

    assert config['status'] == 'unknown'


def test_should_mark_pending_records():
    config = parse_adr_to_config('test/adr/0002-pending-status.md')

    assert config['status'] == 'pending'


def test_should_render_html_with_project_title():
    html = render_html({
        'project_title': 'my-project'
    })

    assert '<title>ADR Viewer - my-project</title>' in html


def test_should_render_html_with_record_status():
    html = render_html({
        'records': [{
            'status': 'accepted',
        }]
    })

    assert '<div class="panel-heading adr-accepted">' in html


def test_should_render_html_with_record_body():
    html = render_html({
        'records': [{
            'body': '<h1>This is my ADR</h1>',
        }]
    })

    assert '<div class="panel-body"><h1>This is my ADR</h1></div>' in html


def test_should_render_html_with_collapsible_index():
    html = render_html({
        'records': [{
            'title': 'Record 123',
            'index': 123
        }]
    })

    assert '<a data-toggle="collapse" href="#collapse123">Record 123</a>' in html
